from django.contrib import admin

from .models import DataChunk


@admin.register(DataChunk)
class DataChunkAdmin(admin.ModelAdmin):
    list_display = ('content_type', 'object_id', 'field_name', 'sequence')
    list_filter = ('content_type', 'field_name')
    search_fields = ('content',)



from django.contrib import admin

# Register your models here.
from django.contrib import admin
from .models import MyModel


class MyModelAdmin(admin.ModelAdmin):
    list_display = ('name', 'content', 'json_data')
    search_fields = ('name',)
    list_filter = ('name',)


admin.site.register(MyModel, MyModelAdmin)



# serializers.py
from rest_framework import serializers
from .models import MyModel

class MyModelSerializer(serializers.ModelSerializer):
    class Meta:
        model = MyModel
        fields = '__all__'


import json

from rest_framework import serializers


class ChunkedJSONField(serializers.Field):
    def to_representation(self, value):
        # `value` here is the raw string returned by the chunked field.
        if value is None or value.strip() == '':
            return None
        return json.loads(value)

    def to_internal_value(self, data):
        # `data` here is the incoming Python object from the request.
        # Convert it to a JSON string to store in the chunked field.
        return json.dumps(data)



from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models


class DataChunk(models.Model):
    """
    Model to store chunks of data for a field in a Django model.

    Attributes:
        content_type (ForeignKey): Reference to the ContentType of the model.
        object_id (PositiveIntegerField): ID of the model instance.
        field_name (CharField): Name of the field being chunked.
        sequence (PositiveIntegerField): Sequence number of the chunk.
        content (CharField): The actual chunk of data.
        content_object (GenericForeignKey): Generic foreign key to the model instance.
    """

    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    object_id = models.PositiveIntegerField()
    field_name = models.CharField(max_length=100)
    sequence = models.PositiveIntegerField()
    content = models.CharField(max_length=4000)

    content_object = GenericForeignKey('content_type', 'object_id')

    class Meta:
        unique_together = ('content_type', 'object_id', 'field_name', 'sequence')
        ordering = ['sequence']

    def __str__(self):
        return f"Chunk {self.sequence} for {self.content_object}"



# Generated by Django 3.2 on 2024-12-03 14:01

import chunked_field.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='MyModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('content', chunked_field.fields.ChunkedTextField()),
                ('json_data', chunked_field.fields.ChunkedTextField()),
            ],
        ),
    ]



# Generated by Django 3.2 on 2024-12-03 14:01

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='DataChunk',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_id', models.PositiveIntegerField()),
                ('field_name', models.CharField(max_length=100)),
                ('sequence', models.PositiveIntegerField()),
                ('content', models.CharField(max_length=4000)),
                ('content_type',
                 models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype')),
            ],
            options={
                'ordering': ['sequence'],
                'unique_together': {('content_type', 'object_id', 'field_name', 'sequence')},
            },
        ),
    ]



from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework import status

from demo.models import MyModel
from demo.serializers import MyModelSerializer


class MyModelAPIView(APIView):
    def get(self, request, *args, **kwargs):
        mymodels = MyModel.objects.all()
        serializer = MyModelSerializer(mymodels, many=True)
        return Response(serializer.data)

    def post(self, request, *args, **kwargs):
        serializer = MyModelSerializer(data=request.data)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data, status=status.HTTP_201_CREATED)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)



import os

from setuptools import find_packages, setup


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name='django-chunked-field',
    version='0.0.3',
    description='A Django app providing a ChunkedTextField for storing large text data in chunks.',
    long_description=read('README.md'),
    long_description_content_type='text/markdown',
    author='berkayeren',
    author_email='berkay-eren@hotmail.com',
    url='https://github.com/berkayeren/django-chunked-field',
    packages=find_packages(),
    include_package_data=True,
    license='MIT License',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 3.2',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
    ],
    install_requires=[
        'Django>=3.2',
        'djangorestframework>=3.14.0'
    ],
)



"""
This module defines a custom Django model field `ChunkedTextField` that stores data in chunks to bypass database limitations.
It includes signal handlers to manage the chunked data during model instance lifecycle events.
"""

from typing import Any, Optional

from django import forms
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.db.models.signals import post_save, post_delete, post_init
from django.utils.text import Truncator

from .models import DataChunk


class ChunkedTextField(models.TextField):
    """
    A custom Django TextField that stores data in chunks to bypass database limitations.
    """

    description = "Text field that stores data in chunks to bypass database limitations."

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        self.truncate_length = kwargs.pop('truncate_length', 50)  # Default truncate length
        self.chunk_size = kwargs.pop('truncate_length', 4000)  # Default chunk size
        kwargs['editable'] = True  # Ensure the field is editable
        super().__init__(*args, **kwargs)
        self.concrete = False  # Make sure Django knows this is not a concrete field
        self.column = None  # Make sure Django knows this is not a concrete field

    def __str__(self) -> str:
        return self.name

    def contribute_to_class(self, cls: Any, name: str, **kwargs: Any) -> None:
        """
        Contribute the field to the model class and connect signal handlers.
        """
        super().contribute_to_class(cls, name, **kwargs)
        setattr(cls, self.name, self.Descriptor(self))
        post_save.connect(self.post_save, sender=cls)
        post_delete.connect(self.post_delete, sender=cls)
        post_init.connect(self.post_init, sender=cls)

    def post_init(self, instance: Any, **kwargs: Any) -> None:
        """
        Signal handler to set the field's value from chunks after the model instance is initialized.
        """
        if instance.pk is not None:
            value = self.get_chunked_value(instance)
        else:
            value = getattr(instance, self.attname)
        setattr(instance, self.attname, value)

    def post_save(self, instance: Any, **kwargs: Any) -> None:
        """
        Signal handler to save chunks after the main model instance is saved.
        """
        value = getattr(instance, self.attname)
        if value is None:
            value = ''
            if instance._chunked_field_cache.get(self.name) is not None:
                value = instance._chunked_field_cache[self.name]
        else:
            value = str(value)

        ct = ContentType.objects.get_for_model(instance, for_concrete_model=False)
        DataChunk.objects.filter(
            content_type=ct,
            object_id=instance.pk,
            field_name=self.name
        ).delete()

        chunk_size = self.chunk_size
        chunks = [value[i:i + chunk_size] for i in range(0, len(value), chunk_size)]

        DataChunk.objects.bulk_create([
            DataChunk(
                content_type=ct,
                object_id=instance.pk,
                field_name=self.name,
                sequence=idx,
                content=chunk
            )
            for idx, chunk in enumerate(chunks)
        ])

        if hasattr(instance, '_chunked_field_cache') and self.name in instance._chunked_field_cache:
            del instance._chunked_field_cache[self.name]

    def post_delete(self, instance: Any, **kwargs: Any) -> None:
        """
        Signal handler to delete chunks when the main model instance is deleted.
        """
        ct = ContentType.objects.get_for_model(instance, for_concrete_model=False)
        DataChunk.objects.filter(
            content_type=ct,
            object_id=instance.pk,
            field_name=self.name
        ).delete()

    def get_chunked_value(self, instance: Any) -> str:
        """
        Retrieve chunks and assemble the value.
        """
        if instance.pk is None:
            return ''
        ct = ContentType.objects.get_for_model(instance, for_concrete_model=False)
        chunks = DataChunk.objects.filter(
            content_type=ct,
            object_id=instance.pk,
            field_name=self.name
        ).order_by('sequence').values_list('content', flat=True)
        value = ''.join(chunks)
        return value

    def formfield(self, form_class: Optional[forms.Field] = None, choices_form_class: Optional[forms.Field] = None,
                  **kwargs: Any) -> forms.Field:
        """
        Return a Django form field for this model field.
        """
        defaults = {'widget': forms.Textarea}
        defaults.update(kwargs)
        return super().formfield(**defaults)

    def db_type(self, connection: Any) -> Optional[str]:
        """
        Return None as this field does not have a direct database representation.
        """
        return 'varchar(1)'

    def get_prep_value(self, value: Any) -> str:
        """
        Return an empty string or placeholder to store in the database.
        """
        return ''

    def from_db_value(self, value: Any, expression: Any, connection: Any) -> str:
        """
        Return an empty string; the real value is set in post_init.
        """
        return ''

    def value_to_string(self, obj: Any) -> str:
        """
        Return the truncated value for serialization and display.
        """
        value = self.value_from_object(obj)
        return Truncator(value).chars(self.truncate_length, truncate='...')

    class Descriptor:
        """
        Descriptor class to manage access to the chunked field value.
        """

        def __init__(self, field: 'ChunkedTextField') -> None:
            self.field = field

        def __get__(self, instance: Any, owner: Any) -> Any:
            """
            Retrieve the chunked value from the cache or database.
            """
            if instance is None:
                return self

            if not hasattr(instance, '_chunked_field_cache'):
                instance._chunked_field_cache = {}
            if self.field.name in instance._chunked_field_cache:
                return instance._chunked_field_cache[self.field.name]
            if instance.pk is None:
                return instance._chunked_field_cache.get(self.field.name, '')

            ct = ContentType.objects.get_for_model(instance, for_concrete_model=False)
            value_chunks = DataChunk.objects.filter(
                content_type=ct,
                object_id=instance.pk,
                field_name=self.field.name
            ).order_by('sequence').values_list('content', flat=True)

            value = ''.join(value_chunks)
            instance._chunked_field_cache[self.field.name] = value
            return value

        def __set__(self, instance: Any, value: Any) -> None:
            """
            Set the chunked value in the instance's dictionary and cache.
            """
            instance.__dict__[self.field.attname] = value
            if not hasattr(instance, '_chunked_field_cache'):
                instance._chunked_field_cache = {}
            instance._chunked_field_cache[self.field.name] = value



from django.db import models

from django.db import models
from chunked_field.fields import ChunkedTextField


class MyModel(models.Model):
    name = models.CharField(max_length=255)
    content = ChunkedTextField(truncate_length=100)  # Adjust truncate_length as needed
    json_data = ChunkedTextField()

    def __str__(self):
        return self.name
